/*
    *
    * This file is a part of CoreUniverse.
    * Shows releated information of apps from CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <QAbstractButton>
#include <QPushButton>
#include <QFile>
#include <QFontDatabase>
#include <QMessageBox>
#include <QScroller>
#include <QDesktopServices>
#include <QProcess>

#include <cprime/cprime.h>

#include "coreuniverse.h"
#include "ui_coreuniverse.h"


coreuniverse::coreuniverse(QWidget *parent): QWidget(parent), ui(new Ui::coreuniverse)
{
    ui->setupUi(this);
    ui->suiteNameVersion->setText("C Suite v" + QString(VERSION_TEXT));

    QPalette pltt(palette());
    pltt.setColor(QPalette::Base, Qt::transparent);
    setPalette(pltt);

    loadSettings();
    startSetup();

    on_about_clicked();
}

coreuniverse::~coreuniverse()
{
    delete ui;
}

/**
 * @brief Setup ui elements
 */
void coreuniverse::startSetup()
{
    QFont font;
    font.setFamily(QFontDatabase::systemFont(QFontDatabase::GeneralFont).family());
    this->setFont(font);

    QTextCursor c = ui->pgother->textCursor();
    QTextCharFormat format = c.charFormat();

    QFont fontyour = font;
    format.setFont(fontyour);
    c.setCharFormat(format);

    // set stylesheet
    ui->sideView->setStyleSheet(CPrime::ThemeFunc::getSideViewStyleSheet());
    ui->sideView->setFixedWidth(sideViewSize);

    // all toolbuttons icon size in sideView
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(sideViewIconSize);
        }
    }

    ui->menu->setVisible(0);
    ui->appTitle->setAttribute(Qt::WA_TransparentForMouseEvents);
    ui->appTitle->setFocusPolicy(Qt::NoFocus);
    this->resize(800, 500);

    if (uiMode != 0) {
        this->setWindowState(Qt::WindowMaximized);
        QScroller::grabGesture(ui->pgother, QScroller::LeftMouseButtonGesture);

        if (uiMode == 2) {
            ui->menu->setIconSize(toolBarIconSize);

            connect(ui->menu, &QToolButton::clicked, this, &coreuniverse::showSideView);
            connect(ui->appTitle, &QToolButton::clicked, this, &coreuniverse::showSideView);

            ui->sideView->setVisible(0);
            ui->menu->setVisible(1);
        }

    }
}

/**
 * @brief Loads application settings
 */
void coreuniverse::loadSettings()
{
    // Sideview is visible in default
    sideView = 1;

    sideViewSize = sm->value("CoreApps", "SideViewSize");
    sideViewIconSize = sm->value("CoreApps", "SideViewIconSize");
    toolBarIconSize = sm->value("CoreApps", "ToolBarIconSize");
    uiMode = sm->value("CoreApps", "UIMode");
}

void coreuniverse::pageClick(QToolButton *btn, int i, QString title)
{
    // first do this to get all the space for contents
    if (uiMode == 2){
        ui->sideView->setVisible(0);
    }

    // all button checked false
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        b->setChecked(false);
    }

    btn->setChecked(true);
    ui->selectedsection->setText(title);
    this->setWindowTitle(title + " - CoreUniverse");
    ui->pages->setCurrentIndex(i);

}

void coreuniverse::showSideView()
{
    if (uiMode == 2) {
        if (ui->sideView->isVisible()) {
            ui->sideView->setVisible(0);
        } else {
            ui->sideView->setVisible(1);
        }
    }
}
void coreuniverse::on_releaseNotes_clicked()
{
    pageClick(ui->releaseNotes, 1, tr("Release Notes"));
    QFile p(":/docs/ReleaseNotes");
    p.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream o(&p);
    ui->pgother->setText(o.readAll());
    p.close();
}

void coreuniverse::on_about_clicked()
{
    pageClick(ui->about, 0, tr("About"));
}

void coreuniverse::on_credits_clicked()
{
    pageClick(ui->credits, 1, tr("Credits"));
    QString str = QString("Developers.\n"
                          "* Abrar <s96abrar:matrix.org>\n"
                          "* Marcus <marcusbritanicus:matrix.org>\n"
                          "* Saber <rahmanshaber:matrix.org>\n\n");
    ui->pgother->setAcceptRichText(true);
    ui->pgother->setText(str);
}

void coreuniverse::on_recommendation_clicked()
{
    pageClick(ui->recommendation, 1, tr("Recommendation"));
    QString str = QString("A few recommendations, tweaks, and tips & tricks for best experience with CoreApps.\n\n"
                          "* Use Qt5ct to change theme and icons and use Fusion as style.\n"
                          "* Use Kvantum to customize the themes.\n"
                          "* If you see fonts are not right, make a font.config using qt5ct.\n"
                          "* Please delete the coreapps.config file from home folder if you saw any ui or settings issues.\n"
                          "* Checkout other Qt apps from Manjaro's Qt app list from https://wiki.manjaro.org/index.php?title=List_of_Qt_Applications ."
                          "* Use Breeze Icon Theme so there won't be any missing icon in any apps");
    ui->pgother->setText(str);
}

void coreuniverse::on_copyright_clicked()
{
    pageClick(ui->copyright, 1, tr("Copyright"));
    QFile p(":/docs/License");
    p.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream o(&p);
    QString str = QString("The different components are copyrighted by their respective authors.\n\n"
                          "\n"
                          "The 19 Apps packages including this one are distributed under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 or later of the License, or (at your option) any later version.\n"
                          "\n"
                          "\n"
                         );
    ui->pgother->setText(str + o.readAll());
    p.close();
}

void coreuniverse::on_changelog_clicked()
{
    pageClick(ui->changelog, 1, tr("ChangeLog"));
    QFile p(":/docs/ChangeLog");
    p.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream o(&p);
    ui->pgother->setText(o.readAll());
    p.close();
}

void coreuniverse::on_cSuite_clicked()
{
    pageClick(ui->cSuite, 1, tr("App Suite"));
    QFile p(":/docs/about.html");
    p.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream o(&p);
    ui->pgother->setAcceptRichText(true);
    ui->pgother->setHtml(o.readAll());
    p.close();
}

void coreuniverse::on_updateBtn_clicked() {

    QProcess download;
    download.start( "wget", QStringList() << "https://gitlab.com/cubocore/coreapps-edge/raw/master/LATEST" << "-O" << "/tmp/coreapps-LATEST" );
    download.waitForFinished();

    QFile upstream( "/tmp/coreapps-LATEST" );
    upstream.open( QFile::ReadOnly );
    QString upstreamVer = QString::fromLocal8Bit( upstream.readAll() ).trimmed();
    upstream.close();

    QFile local( ":/docs/LATEST" );
    local.open( QFile::ReadOnly );
    QString localVer = QString::fromLocal8Bit( local.readAll() ).trimmed();
    local.close();

    if ( upstreamVer.isEmpty() ) {
        QMessageBox::information(
            this,
            "C Suite - Unable to connect",
            "There seems to be a problem connecting to https://gitlab.com to check for "
            "updates. Please ensure you have a working internet connection and try again."
        );
    }

    else if ( upstreamVer.compare( localVer ) > 0 ) {
        QMessageBox msgBox;
        msgBox.setWindowTitle("C Suite - Update Available");
        msgBox.setText(tr("There is an update available upstream. Please download and install the latest update for the best experience with C Suite.\n"
                          "Click the 'Get Update' button to install it if your OS don't not have it."));

        QAbstractButton *pButtonYes = msgBox.addButton(tr("Get Update"), QMessageBox::YesRole);
        msgBox.addButton(tr("Nope"), QMessageBox::NoRole);
        msgBox.exec();

        if (msgBox.clickedButton() == pButtonYes) {
            QString link = "https://gitlab.com/cubocore/coreapps-edge/";
            QDesktopServices::openUrl(QUrl(link));
        }
    }

    else {
        QMessageBox::information(
            this,
            "CuboCore - Latest",
            "Hooray! You have the latest version available. Should you have any doubts, problem or suggestions feel free to open a bug report "
            "at https://gitlab.com/groups/cubocore/-/issues"
        );
    }
}
